@@ -478,14 +478,6 @@ def mc_bulk_subparser(subparser):
         help="Path of the output directory, will be created if not exist."
     )
 
-    parser_req.add_argument(
-        "--bigwig_contexts",
-        type=str,
-        nargs='+',
-        required=True,
-        help="mC contexts for generating the bigwig tracks."
-    )
-
     parser_req.add_argument(
         "--chrom_size_path",
         type=str,
@@ -494,34 +486,164 @@ def mc_bulk_subparser(subparser):
     )
 
     parser_opt.add_argument(
-        "--extract_mcg",
-        dest='extract_mcg',
-        action='store_true',
-        help='Whether run the step to extract mCG sites from the merged ALLC. '
-             'If your input ALLC only contains mCG sites, this can be skipped. '
-             'Otherwise, this needs to be done before running the CG-DMR calling.'
-    )
-    parser.set_defaults(extract_mcg=False)
-
-    parser_opt.add_argument(
-        "--bigwig_bin_size",
-        type=int,
-        default=50,
-        help="Bin size used to generate bigwig."
-    )
-
-    parser_opt.add_argument(
-        "--merge_allc_cpu",
-        type=int,
-        default=8,
-        help="Number of CPU to use in individual merge-allc job."
+        "--mch_context",
+        type=str,
+        default='CHN',
+        help="mCH contexts for generating the bigwig tracks."
     )
 
     parser_opt.add_argument(
         "--mcg_context",
         type=str,
         default='CGN',
-        help="mC context for extract_mcg step, only relevant when extract_mcg=True."
+        help="mCG contexts for generating the bigwig tracks and merge strand."
+    )
+
+    parser_opt.add_argument(
+        "--bigwig_mch_bin_size",
+        type=int,
+        default=50,
+        help="Bin size used to generate mCH bigwig."
+    )
+
+    parser_opt.add_argument(
+        "--bigwig_mcg_bin_size",
+        type=int,
+        default=1,
+        help="Bin size used to generate mCG bigwig."
+    )
+
+    parser_opt.add_argument(
+        "--cpu_per_job",
+        type=int,
+        default=12,
+        help="Number of CPUs to use in individual merge-allc job."
+    )
+
+    parser_opt.add_argument(
+        "--total_cpu",
+        type=int,
+        default=12,
+        help="Number of CPUs to use in total."
+    )
+
+
+def snm3c_imputation_subparser(subparser):
+    parser = subparser.add_parser('m3c-impute',
+                                  formatter_class=argparse.ArgumentDefaultsHelpFormatter,
+                                  help="Prepare the snakefile for snm3C-seq contact matrices imputation.")
+    parser_req = parser.add_argument_group("Required inputs")
+    parser_opt = parser.add_argument_group("Optional inputs")
+
+    parser_req.add_argument(
+        "--output_dir",
+        type=str,
+        required=True,
+        help="Path of the output directory after mapping is finished."
+    )
+
+    parser_req.add_argument(
+        "--chrom_size_path",
+        type=str,
+        required=True,
+        help="Path of the chromosome sizes file."
+    )
+
+    parser_opt.add_argument(
+        "--scheduler",
+        type=str,
+        default='sbatch',
+        choices=['sbatch', 'qsub'],
+        help="Which scheduler to use, if using sbatch, remember to provide remote chrom_size_path."
+    )
+
+    parser_opt.add_argument(
+        "--scool_cpu",
+        type=int,
+        default=5,
+        help="Number of CPU to use in generating scool files."
+    )
+
+    parser_opt.add_argument(
+        "--cpu_per_job",
+        type=int,
+        default=10,
+        help="Number of CPU for each imputation job."
+    )
+
+    parser_opt.add_argument(
+        "--batch_size",
+        type=int,
+        default=100,
+        help="Number of cells to process in each imputation job."
+    )
+
+    parser_opt.add_argument(
+        "--total_cpu",
+        type=int,
+        default=100,
+        help="Number of total CPUs to use in qsub."
+    )
+
+    parser_opt.add_argument(
+        "--min_contacts_per_cell",
+        type=int,
+        default=50000,
+        help="Minimum contacts for a cell to be included in the contacts analysis."
+    )
+
+    parser_opt.add_argument(
+        "--sbatch_time_str",
+        type=str,
+        default='2:00:00',
+        help="Time estimation for sbatch jobs."
+    )
+
+    parser_opt.add_argument(
+        "--min_cutoff",
+        type=str,
+        default='1e-5',
+        help="Minimum value cutoff of the imputation matrix. "
+             "Values smaller than the cutoff will be set to 0 to reduce file size."
+    )
+
+    parser_opt.add_argument(
+        "--skip_scool_prep",
+        dest='skip_scool_prep',
+        action='store_true',
+        help='Skip scool preparation step if you already have them prepared in the same location '
+             'and just want to use this command to change settings and update snakefiles.'
+    )
+    parser.set_defaults(skip_scool_prep=False)
+    return
+
+
+def snm3c_dataset_subparser(subparser):
+    parser = subparser.add_parser('m3c-dataset',
+                                  formatter_class=argparse.ArgumentDefaultsHelpFormatter,
+                                  help="Prepare the snakefile for snm3C-seq contact matrices imputation.")
+    parser_req = parser.add_argument_group("Required inputs")
+    parser_opt = parser.add_argument_group("Optional inputs")
+
+    parser_req.add_argument(
+        "--output_dir",
+        type=str,
+        required=True,
+        help="Path of the output directory after mapping is finished."
+    )
+
+    parser_req.add_argument(
+        "--fasta_path",
+        type=str,
+        required=True,
+        help="Path of the chromosome fasta file."
+    )
+
+    parser_opt.add_argument(
+        "--cpu",
+        type=int,
+        default=10,
+        help="Number of CPU for each job."
     )
 
 
@@ -548,6 +670,8 @@ def main():
     start_from_cell_fastq_register_subparser(subparsers)
     summary_register_subparser(subparsers)
     mc_bulk_subparser(subparsers)
+    snm3c_imputation_subparser(subparsers)
+    snm3c_dataset_subparser(subparsers)
 
     # initiate
     args = None
@@ -592,6 +716,10 @@ def main():
         from cemba_data.mapping import final_summary as func
     elif cur_command == 'mc-bulk':
         from cemba_data.bulk import prepare_mc_bulk as func
+    elif cur_command == 'm3c-impute':
+        from cemba_data.snm3C import prepare_impute_dir as func
+    elif cur_command == 'm3c-dataset':
+        from cemba_data.snm3C import prepare_dataset_commands as func
     else:
         log.debug(f'{cur_command} not Known, check the main function if else part')
         parser.parse_args(["-h"])
